import pygame
from .triggeredtext import TriggeredText
from .gate import Gate
from .endlevel import EndLevel
from .actiontext import ActionText
from .picture import Picture

BLOCK_GEOMETRY = BLOCK_WIDTH, BLOCK_HEIGHT = (32, 32)


class SymbolDecoder:
    def __init__(self, level, name):
        self._level = level
        self._name = name
        self._where = level.search(self._name)
        self.components = []

    def add_line(self, line):
        param = line[1:]
        if line[0] == '"':
            self.components.append(TriggeredText(self._where, param))

        if line[0] == '|':
            self.components.append(Gate(self._where, param))

        if line[0] == 'q':
            self.components.append(EndLevel(self._where, param))

        if line[0] == 'i':
            self.components.append(Picture(self._where, param))

        if line[0] == 'a':
            self.components.append(ActionText(self._where, param))

class Level:
    def __init__(self):
        self.lines = []
        self._color = None
        self._current_symbol = None
        self._components = []

    def get_components(self):
        return self._components

    def color(self, color):
        self._color = color
        return self

    def load(self, level_name):
        with open(level_name) as f:
            content = f.readlines()

        read_function = None
        for line in content:
            if line.startswith("START"):
                read_function = lambda line: self._add_line(line)
                continue

            if line.startswith("END"):
                read_function = lambda line: self._add_definition_line(line)
                continue

            if read_function is not None:
                read_function(line)

        self._commit_symbol()

    def _commit_symbol(self):
        if self._current_symbol is not None:
            self._components.extend(self._current_symbol.components)
            self._current_symbol = None

    def _add_definition_line(self, line):
        line = line.strip()
        if len(line) == 0:
            return

        if line[0] == ':':
            self._commit_symbol()
            self._current_symbol = SymbolDecoder(self, line[1:])
            return

        if self._current_symbol is None:
            return

        self._current_symbol.add_line(line)

    def _add_line(self, line):
        self.lines.append(line)

    def search(self, item):
        for y, line in enumerate(self.lines):
            where = line.find(item)
            if where != - 1:
                return (where, y)

        return (-1, -1)

    def render(self, surface):
        for y, line in enumerate(self.lines):
            for x, c in enumerate(line):
                if c == "X":
                    fill_rect = pygame.Rect(x * BLOCK_WIDTH, y * BLOCK_HEIGHT,
                        BLOCK_WIDTH, BLOCK_HEIGHT)
                    surface.fill(self._color, fill_rect)

def position_to_screen(position):
    return (position[0] * BLOCK_WIDTH, position[1] * BLOCK_HEIGHT)

