""" A Mover takes a Movable and act on it.

    A Movable has the move( delta_position_tuple ) method. """

import unittest


class Mover:
    def __init__(self, movable):
        self._movable = movable
        self._movement = (0, 0)
        self._all_movements = set()

    def on_init(self, systems):
        return

    def _apply_movements(self):
        dx, dy = reduce(lambda t1, t2: (t1[0] + t2[0], t1[1] + t2[1]),
                self._all_movements, (0, 0))

        self._movement = (dx, dy)

    def start_move(self, delta):
        self._all_movements.add(delta)
        self._apply_movements()

    def stop_move(self, delta):
        try:
            self._all_movements.remove(delta)
        except KeyError:
            pass
        self._apply_movements()

    def update(self, dt):
        if self._movable is not None:
            _dx, _dy = self._movement
            dt = dt / 1000.
            self._movable.move((_dx * dt, _dy * dt))


class MockMovable:
    def __init__(self):
        self.m = (0, 0)

    def move(self, m):
        self.m = m


class TestMover(unittest.TestCase):
    def setUp(self):
        self.movable = MockMovable()
        self.mover = Mover(self.movable)

    def test_no_move(self):
        self.mover.update(1)

        self.assertEqual(self.movable.m[0], 0.)
        self.assertEqual(self.movable.m[1], 0.)

    def test_add_move(self):
        self.mover.start_move((1, 0))
        self.mover.update(1)

        self.assertEqual(self.movable.m[0], 0.001)
        self.assertEqual(self.movable.m[1], 0.)

    def test_stop_move(self):
        self.mover.start_move((1, 0))
        self.mover.stop_move((1, 0))
        self.mover.update(1)

        self.assertEqual(self.movable.m[0], 0.)
        self.assertEqual(self.movable.m[1], 0.)

    def test_add_move_twice(self):
        self.mover.start_move((1, 0))
        self.mover.start_move((1, 0))
        self.mover.update(1)

        self.assertEqual(self.movable.m[0], 0.001)
        self.assertEqual(self.movable.m[1], 0.)

    def test_stop_move_twice(self):
        self.mover.start_move((1, 0))
        self.mover.stop_move((1, 0))
        self.mover.stop_move((1, 0))
        self.mover.update(1)

        self.assertEqual(self.movable.m[0], 0.)
        self.assertEqual(self.movable.m[1], 0.)

if __name__ == '__main__':
    unittest.main()
